@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package DBIx::Class::ResultSet::RecursiveUpdate;
 
-use version; our $VERSION = qv('0.008');
+use version; our $VERSION = qv('0.009');
 
 use base qw(DBIx::Class::ResultSet);
 
@@ -22,7 +22,8 @@ use Scalar::Util qw( blessed );
 
 sub recursive_update {
     my %params = @_;
-    my ( $self, $updates, $fixed_fields, $object ) = @params{ qw/resultset updates fixed_fields object/ }; 
+    my ( $self, $updates, $fixed_fields, $object, $resolved, $if_not_submitted ) = @params{ qw/resultset updates fixed_fields object resolved if_not_submitted/ }; 
+    $resolved ||= {};
     # warn 'entering: ' . $self->result_source->from();
     carp 'fixed fields needs to be an array ref' if $fixed_fields && ref($fixed_fields) ne 'ARRAY';
     my %fixed_fields;
@@ -30,6 +31,23 @@ sub recursive_update {
     if ( blessed($updates) && $updates->isa('DBIx::Class::Row') ) {
         return $updates;
     }
+    if ( $updates->{id} ){
+        $object = $self->find( $updates->{id}, { key => 'primary' } );
+    }
+    my @missing =
+      grep { !exists $updates->{$_} && !exists $fixed_fields{$_} } $self->result_source->primary_columns;
+    if ( !$object && !scalar @missing ) {
+#        warn 'finding by: ' . Dumper( $updates ); use Data::Dumper;
+        $object = $self->find( $updates, { key => 'primary' } );
+    }
+    $updates = { %$updates, %$resolved };
+    @missing =
+      grep { !exists $resolved->{$_} } @missing;
+    if ( !$object && !scalar @missing ) {
+#        warn 'finding by +resolved: ' . Dumper( $updates ); use Data::Dumper;
+        $object = $self->find( $updates, { key => 'primary' } );
+    }
+    $object ||= $self->new( {} );
     # warn Dumper( $updates ); use Data::Dumper;
     # direct column accessors
     my %columns;
@@ -43,22 +61,11 @@ sub recursive_update {
     my %post_updates;
     my %other_methods;
     my %columns_by_accessor = _get_columns_by_accessor( $self );
+#    warn 'resolved: ' . Dumper( $resolved );
+#    warn 'updates: ' . Dumper( $updates ); use Data::Dumper;
+#    warn 'columns: ' . Dumper( \%columns_by_accessor );
     for my $name ( keys %$updates ) {
         my $source = $self->result_source;
-        if( $name eq 'id'
-#            && scalar @{$source->primary_columns} == 1
-            && !$source->has_column( 'id' )
-        ){
-            my @ids = ( $updates->{id} );
-            if( ref $updates->{id} ){
-                @ids = @{ $updates->{id} };
-            }
-            my $i = 0;
-            for my $key ( $source->primary_columns ){
-                $columns{ $key } = $ids[ $i++ ];
-            }
-            next;
-        }
         if ( $columns_by_accessor{$name}
             && !( $source->has_relationship($name) && ref( $updates->{$name} ) )
           )
@@ -85,12 +92,6 @@ sub recursive_update {
     }
     # warn 'other: ' . Dumper( \%other_methods ); use Data::Dumper;
 
-    my @missing =
-      grep { !exists $columns{$_} && !exists $fixed_fields{$_} } $self->result_source->primary_columns;
-    if ( !$object && !scalar @missing ) {
-        $object = $self->find( \%columns, { key => 'primary' } );
-    }
-    $object ||= $self->new( {} );
     # first update columns and other accessors - so that later related records can be found
     for my $name ( keys %columns ) {
         $object->$name( $columns{$name} );
@@ -100,13 +101,15 @@ sub recursive_update {
     }
     for my $name ( keys %pre_updates ) {
         my $info = $object->result_source->relationship_info($name);
-        _update_relation( $self, $name, $updates, $object, $info );
+        _update_relation( $self, $name, $updates, $object, $info, $if_not_submitted );
     }
 #    $self->_delete_empty_auto_increment($object);
 # don't allow insert to recurse to related objects - we do the recursion ourselves
 #    $object->{_rel_in_storage} = 1;
+
     $object->update_or_insert;
 
+
     # updating many_to_many
     for my $name ( keys %$updates ) {
         next if exists $columns{$name};
@@ -141,7 +144,7 @@ sub recursive_update {
     }
     for my $name ( keys %post_updates ) {
         my $info = $object->result_source->relationship_info($name);
-        _update_relation( $self, $name, $updates, $object, $info );
+        _update_relation( $self, $name, $updates, $object, $info, $if_not_submitted );
     }
     return $object;
 }
@@ -159,7 +162,7 @@ sub _get_columns_by_accessor {
 }
 
 sub _update_relation {
-    my ( $self, $name, $updates, $object, $info ) = @_;
+    my ( $self, $name, $updates, $object, $info, $if_not_submitted ) = @_;
     my $related_result =
       $self->related_resultset($name)->result_source->resultset;
     my $resolved;
@@ -171,20 +174,32 @@ sub _update_relation {
     }
 
  #                    warn 'resolved: ' . Dumper( $resolved ); use Data::Dumper;
-    $resolved = undef
+    $resolved = {}
       if defined $DBIx::Class::ResultSource::UNRESOLVABLE_CONDITION && $DBIx::Class::ResultSource::UNRESOLVABLE_CONDITION == $resolved;
     if ( ref $updates->{$name} eq 'ARRAY' ) {
+        my @updated_ids;
         for my $sub_updates ( @{ $updates->{$name} } ) {
-            $sub_updates = { %$sub_updates, %$resolved } if $resolved && ref( $sub_updates ) eq 'HASH';
             my $sub_object =
-              recursive_update( resultset => $related_result, updates => $sub_updates );
+              recursive_update( resultset => $related_result, updates => $sub_updates, resolved => $resolved );
+            push @updated_ids, $sub_object->id;
+        }
+        my @related_pks = $related_result->result_source->primary_columns;
+        if( defined $if_not_submitted && $if_not_submitted eq 'delete' ){
+            if ( 1 == scalar @related_pks ){
+                $object->$name->search( { $related_pks[0] => { -not_in => \@updated_ids } } )->delete;
+            }
+        }
+        elsif( defined $if_not_submitted && $if_not_submitted eq 'set_to_null' ){
+            if ( 1 == scalar @related_pks ){
+                my @fk = keys %$resolved;
+                $object->$name->search( { $related_pks[0] => { -not_in => \@updated_ids } } )->update( { $fk[0] => undef } );
+            }
         }
     }
     else {
         my $sub_updates = $updates->{$name};
         my $sub_object;
         if( ref $sub_updates ){
-            $sub_updates = { %$sub_updates, %$resolved } if $resolved && ref( $sub_updates ) eq 'HASH';
             # for might_have relationship
             if( $info->{attrs}{accessor} eq 'single' && defined $object->$name ){
                 $sub_object = recursive_update( 
@@ -195,7 +210,7 @@ sub _update_relation {
             }
             else{ 
                 $sub_object =
-                recursive_update( resultset => $related_result, updates => $sub_updates );
+                recursive_update( resultset => $related_result, updates => $sub_updates, resolved => $resolved );
             }
         }
         elsif( ! ref $sub_updates ){
